/*
 *  Fireworks OpenGL Demonstrator
 *  Moving parallel or spot light with textures and ambient light control
 *
 *  Created by brisson on 2012/05/17.
 *  Updated by brisson on 2013/05/04.
 *  Copyright (c) 2012-2013 J M. Brisson Lopes & Instituto Superior Tecnico. All rights reserved.
 *
 */
/*******************************************************************************
 * Fireworks - an OpenGL program to simulate fireworks explosions              *
 * Copyright (C) 2012, 2013 IST & J M Brisson Lopes                            *
 *                                                                             *
 *  This software is provided 'as-is', without any express or implied          *
 *  warranty.  In no event will the authors be held liable for any damages     *
 *  arising from the use of this software.                                     *
 *                                                                             *
 *  Permission is granted to anyone to use this software for any purpose,      *
 *  including commercial applications, and to alter it and redistribute it     *
 *  freely, subject to the following restrictions:                             *
 *                                                                             *
 *  1. The origin of this software must not be misrepresented; you must not    *
 *     claim that you wrote the original software. If you use this software    *
 *     in a product, an acknowledgment in the product documentation would be   *
 *     appreciated but is not required.                                        *
 *  2. Altered source versions must be plainly marked as such, and must not be *
 *     misrepresented as being the original software.                          *
 *  3. This notice may not be removed or altered from any source distribution. *
 *                                                                             *
 * The "official" glbmp webpage is: http://chaoslizard.sourceforge.net/glbmp/  *
 ******************************************************************************/

#pragma once

#include <string>
#include <list>
using std::string;
using std::list;


// Particle class **************************************************************

class Particle {
	float	life;		// life
	float	fade;		// fade
	float	 r,  g,  b; // colour
	float    x,  y,  z; // position
	float   vx, vy, vz; // speed
    float   ax, ay, az; // aceleration
    static unsigned int ParticleSkeleton;
   
public:
    Particle();
    Particle( const Particle & another) { *this = another; }
    const Particle & operator = ( const Particle & another);
    const bool Update( const float elapsedTime, const bool respawn);
    void       Draw() const;
    const bool Respawn( const bool force = false);
	const bool IsAlive() const { return life>0.f; }
	const bool IsDead()  const { return life<=0.0f; } 
    
    static unsigned int LoadTexture( const string & textureBitmapFilename);
    static unsigned int CreateParticleSkeleton();
};

// Particle system *************************************************************

class ParticleSystem {
    
public:
    ParticleSystem( const bool burst = true);
    void Update( const float elapsed);
    void Draw() const;
    void Reset();
	const int NoParticles() const { return (int) activeParticles.size(); }
protected:
    list<Particle*> activeParticles, deadParticles;
    bool              respawnParticles;
    static int        Max_Particles;
};

// Camera class ****************************************************************

class Camera {
    
public:
    Camera();
    void Projection() const;
protected:
    static float CameraFOV;
};


//  The Application class *****************************************************

class OpenGLApplication {
    
public:
    OpenGLApplication();
    static void Initialize( int argc, char ** argv) { MyFireworks.InitApplication( argc, argv); }
protected:
    bool  Fullscreen, ShowHUD, TimerOn;
    int   lastTime;
    int   frameConter, lastFramerateUpdateTime;
    float frameRate;
    
    Camera         * MyCamera;
    ParticleSystem * MySystem;
    
    void InitApplication( int argc, char ** argv);
    void MyReshape( const int width, const int height);
    void MyDisplay();
    void MyKeyboard( const unsigned char key, const int x, const int y);
    void MySpecialKeys( const int key, const int x, const int y);
    void MyTimer( const int value);
    void MyIdle();
    void HUD() const;
    void DisplayString( const int x, const int y, const string & label, const void * font = NULL) const;
    
    static OpenGLApplication MyFireworks;
    
    static void reshape( int w, int h) { MyFireworks.MyReshape( w, h); }
    static void display() { MyFireworks.MyDisplay(); }
    static void keyboard( unsigned char key, int x, int y) { MyFireworks.MyKeyboard( key, x, y); }
    static void special( int key, int x, int y) { MyFireworks.MySpecialKeys( key, x, y); }
    static void timer( int value) { MyFireworks.MyTimer( value); }
    static void idle() { MyFireworks.MyIdle(); }
    
    static int  timerInterval;
    
    static string CommandLabel;
};
