/*******************************************************************************
 * Fireworks - an OpenGL program to simulate fireworks explosions              *
 * Copyright (C) 2012 IST                                                      *
 *                                                                             *
 *  This software is provided 'as-is', without any express or implied          *
 *  warranty.  In no event will the authors be held liable for any damages     *
 *  arising from the use of this software.                                     *
 *                                                                             *
 *  Permission is granted to anyone to use this software for any purpose,      *
 *  including commercial applications, and to alter it and redistribute it     *
 *  freely, subject to the following restrictions:                             *
 *                                                                             *
 *  1. The origin of this software must not be misrepresented; you must not    *
 *     claim that you wrote the original software. If you use this software    *
 *     in a product, an acknowledgment in the product documentation would be   *
 *     appreciated but is not required.                                        *
 *  2. Altered source versions must be plainly marked as such, and must not be *
 *     misrepresented as being the original software.                          *
 *  3. This notice may not be removed or altered from any source distribution. *
 *                                                                             *
 * The "official" glbmp webpage is: http://chaoslizard.sourceforge.net/glbmp/  *
 ******************************************************************************/

/*******************************************************************************
 * changes                                                                     *
 * -------                                                                     *
 * 2012-05-17/JB - Adapted to VisualStudio.                                    *
 *                 Made more C++ compliant (C++ streams, C includes).          *
 *                 On-screen instructions added.                               *
 ******************************************************************************/


#if defined(__APPLE__) || defined(MACOSX)
#include <GLUT/glut.h>
#else
#include <GL/glut.h>
#endif

#include <iostream>
//#include <cstdlib>
#include <cmath>
#include "glbmp.h"  //http://chaoslizard.sourceforge.net/glbmp/

using std::cerr;
using std::endl;

#define frand()			((float)rand()/RAND_MAX)
#ifndef M_PI
	#define M_PI 3.14159265
#endif
#define MAX_PARTICULAS  1500

char _instr[] = "e-new explosion, Esc-exit";

int _w, _h;

typedef struct {
	float	life;		// vida
	float	fade;		// fade
	float	r, g, b;    // color
	GLfloat x, y, z;    // posio
	GLfloat vx, vy, vz; // velocidade 
    GLfloat ax, ay, az; // acelerao
} Particle;

Particle particula[MAX_PARTICULAS];
GLuint texname;


/* Loads the specified bitmap file from disk and copies it into an OpenGL texture.
* Returns the GLuint representing the texture (calls exit(1) if the bitmap fails to load).*/

void LoadTexture(const char * bitmap_file)
{
		
		glbmp_t bitmap;     //object to fill with data from glbmp
		
		//try to load the specified file--if it fails, dip out
		if(!glbmp_LoadBitmap(bitmap_file, 0, &bitmap))
		{
			cerr << "Error loading bitmap file: " << bitmap_file << endl;
			exit(1);
		}

	glGenTextures(1, &texname);
	glBindTexture(GL_TEXTURE_2D, texname);
	//copy data from bitmap into texture
	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, bitmap.width, bitmap.height,
				 0, GL_RGB, GL_UNSIGNED_BYTE, bitmap.rgb_data);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
    glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE); /* no era necessrio pois este comando  executado por omisso */
	//free the bitmap
	glbmp_FreeBitmap(&bitmap);
}

void iniParticulas(void)
{
 GLfloat v, theta, phi;
 int i;

 for(i=0; i<MAX_PARTICULAS; i++)
    {
	v = 0.8*frand()+0.2;
    phi = frand()*M_PI;
    theta = 2.0*frand()*M_PI;
    particula[i].x = 0.0f;
    particula[i].y = 15.0f;
    particula[i].z = 0.0f;
    particula[i].vx = v * cos(theta) * sin(phi);
    particula[i].vy = v * cos(phi);
    particula[i].vz = v * sin(theta) * sin(phi);
	particula[i].ax =  0.01f; /* simular um pouco de vento */
    particula[i].ay = -0.15f; /* simular a acelerao da gravidade */
    particula[i].az =  0.0f;
	
	/* tom amarelado que vai ser multiplicado pela textura que varia entre branco e preto */
	particula[i].r = 0.882f;
	particula[i].g = 0.552f;	
	particula[i].b = 0.211f;

	particula[i].life = 1.0f;		/* vida inicial */                
	particula[i].fade = 0.005f;	    /* step de decrscimo da vida para cada iterao */
	}
}

void iniOpenGL(void)
{
 glClearColor(0.0f,0.0f,0.0f,0.0f);      
 glDisable(GL_DEPTH_TEST); /* no interessa o z-buffer: as partculas podem ser desenhadas umas por cima das outras sem problemas de ordenao */
 glEnable(GL_BLEND);	        		
// glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);	
 glBlendFunc(GL_SRC_ALPHA,GL_ONE);
 glHint(GL_PERSPECTIVE_CORRECTION_HINT,GL_NICEST);
 LoadTexture("particula.bmp");
 iniParticulas();
}

void display(void)
{
 int i;

 glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

 glMatrixMode(GL_PROJECTION);
 glLoadIdentity();
 gluPerspective(45.0f,(GLfloat)_w/(GLfloat)_h,0.1f,400.0f);
 glMatrixMode(GL_MODELVIEW);
 glLoadIdentity();
 gluLookAt (0.0, 0.0, 80.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0);

 glEnable(GL_TEXTURE_2D);
 glBindTexture(GL_TEXTURE_2D, texname);
 for (i=0; i<MAX_PARTICULAS; i++)
	{
		if(particula[i].life > 0.0f) /* s desenha as que ainda esto vivas */
		{

			/* A vida da partcula representa o canal alpha da cor. Como o blend est activo a cor final  a soma da cor rgb do fragmento multiplicada pelo
			alpha com a cor do pixel destino */

			glColor4f(particula[i].r,particula[i].g,particula[i].b, particula[i].life);

			glBegin(GL_TRIANGLE_STRIP);	/* 4 vrtices - 2 triangulos: v0 v1 v2; v1 v2 v3 */			        
			glTexCoord2d(1,1); 
			glVertex3f(particula[i].x+1.0f,particula[i].y+1.0f,particula[i].z);            
			glTexCoord2d(0,1); 
			glVertex3f(particula[i].x-1.0f,particula[i].y+1.0f,particula[i].z);       
			glTexCoord2d(1,0); 
			glVertex3f(particula[i].x+1.0f,particula[i].y-1.0f,particula[i].z);        
			glTexCoord2d(0,0); 
			glVertex3f(particula[i].x-1.0f,particula[i].y-1.0f,particula[i].z);      
			glEnd();	
		}
	}
	glDisable( GL_TEXTURE_2D);
	
	glMatrixMode( GL_PROJECTION);
	glLoadIdentity();
	gluOrtho2D( 0., _w, 0., _h);
	glMatrixMode( GL_MODELVIEW);
	glLoadIdentity();
	glColor3f( 1., 1., 0.);
	glRasterPos2d( 5, 5);
	for ( i=0; _instr[i]!='\0'; i++)
		glutBitmapCharacter( GLUT_BITMAP_HELVETICA_12, _instr[i]);
	
 glutSwapBuffers();
}

void iterate(int value)
{
	int i;
	float h; 

	/* Mtodo de Euler de integrao de eq. diferenciais ordinrias 
		h representa o step de tempo; dv/dt = a; dx/dt = v; e conhecem-se os valores iniciais de x e v */

	h=0.125f;
	for (i=0; i<MAX_PARTICULAS; i++)
	{
		particula[i].x += (h*particula[i].vx);
		particula[i].y += (h*particula[i].vy);
		particula[i].z += (h*particula[i].vz);
		particula[i].vx += (h*particula[i].ax);
		particula[i].vy += (h*particula[i].ay);
		particula[i].vz += (h*particula[i].az);
		particula[i].life -= particula[i].fade;
	}
 glutPostRedisplay();
 glutTimerFunc(33,iterate,1);
}

void reshape(int w, int h)
{
 if (!h)
    return;
 glViewport(0, 0,  (GLsizei) w, (GLsizei) h);
 _w = w, _h = h;
}

void keyboard(unsigned char key, int x, int y)
{
 switch (key)
   {
	case 'e':
	iniParticulas();
	break;
	case 27: exit(0);
    break;
   }
}

int main(int argc, char **argv)
{
 glutInit(&argc, argv);
 glutInitDisplayMode (GLUT_DOUBLE | GLUT_RGB);
 glutInitWindowSize (400, 400);
 glutInitWindowPosition (0, 0);
 glutCreateWindow ("Sistema de Particulas em OpenGL");
 iniOpenGL();
 glutDisplayFunc(display);
 glutReshapeFunc(reshape);
 glutKeyboardFunc(keyboard);
 glutTimerFunc(33,iterate,1);
 glutMainLoop();
 return 0;
}

